﻿/*
Автор: Veretragna
Скрипт для экспорта отдельных страниц документа InDesign в формат PDF (Print) с возможностью именования страниц, с поддержкой профилей и указания размеров страницы в имени файла.

Работа скрипта подразумевает частую смену профилей, поэтому используется графический интерфейс.
Заложена возможность экспортировать все именованные страницы документа InDesign в один объединенный документ PDF.
Заложена возможность провести экспорт в любой комбинации режимов для всех открытых документов.

Инструкция по использованию: 
* Выделить нужную страницу документа с помощью инструмента Page Tool и подписать страницу в панели Script Label - это будет именем страницы при экспорте (повторить необходимое количество раз);
* Запустить скрипт;
* Выбрать папку, в которую будут помещаться отдельные страницы и/или объединенные документы;
* Выбрать профиль PDF, который будет использован для экспорта отдельных именованных страниц (если требуется);
* Выбрать профиль PDF, который будет использован для экспорта объединенного документа (если требуется);
* Отметить нужные опции
* Запустить экспорт кнопкой "Старт".

В результате работы скрипта в выбранной папке экспорта будут созданы страницы документа в формате PDF.
Пример имени файла после экспорта: "Страница такая-то - 120x150mm.pdf", "Документ такой-то - merged.pdf"

Особенности скрипта:
* Если для экспорта отдельных именованных страниц или объединенного документа будет использован профиль PDF "[NONE]", скрипт не будет выполнять эту операцию.
* Объединенный документ будет назван по имени родительского документа InDesign с добавлением слова merged. Размеры страниц не включаются.

Пользуйтесь на здоровье.
*/

#targetengine "mine"

polyfillForEach(); // Mozilla polyfill adding .forEach() method to plain JS arrays

var previous = getPrevious();

var v_Margin = previous.v_Margin ? previous.v_Margin : 0;
var v_Overwrite = previous.v_Overwrite ? previous.v_Overwrite : 0;
var v_Merge = previous.v_Merge ? previous.v_Merge : 0;
var v_AllDocs = previous.v_AllDocs ? previous.v_AllDocs : 0;
var mainPDFExportPreset = previous.mainPDFExportPreset ? previous.mainPDFExportPreset : "[NONE]";
var mergePDFExportPreset = previous.mergePDFExportPreset ? previous.mergePDFExportPreset : "[NONE]";
var exportPath = previous.exportPath ? previous.exportPath : "";

var v_Ready = 0;

GUILogic();

function GUILogic() {
	var box = new Window('palette', "[ID] Page Exporter: PDF v1");
	
	box.panel_export = box.add('panel', undefined, "Путь экспорта");
	
	var inputBox = box.panel_export.add("edittext");
	var cg = String(ClipGet());
	var f = Folder(cg);
	if (f.exists)
		inputBox.text = cg;
	else
		inputBox.text = exportPath;
	setBaseProps(inputBox);
	
	var selectFolderButton = box.panel_export.add('button', undefined, "Выбрать папку экспорта", "");
	selectFolderButton.preferredSize = [400, 20];
	selectFolderButton.onClick = function () {
		var defaultPath = new Folder(inputBox.text);
		promptPath = defaultPath.selectDlg("Выберите папку, в которой нужно выполнить поиск линков");
		if (!promptPath)
			return;
		inputBox.text = promptPath;
		exportPath = promptPath;
	}
	
	var c_Overwrite = box.add("checkbox", undefined, "Перезапись старых файлов");
	c_Overwrite.onClick = function () { c_Overwrite.value ? v_Overwrite = 1 : v_Overwrite = 0; }
	if (v_Overwrite == 1) c_Overwrite.value = true;
	setBaseProps(c_Overwrite);

	box.panel_options = box.add('panel', undefined, "Экспорт отдельных страниц поименно");
	var staticCaption = box.panel_options.add('statictext', undefined, "Профиль экспорта отдельных страниц:");
	setBaseProps(staticCaption);
	var exportPresets = box.panel_options.add('listbox', undefined, refineDomArray(app.pdfExportPresets));
	exportPresets.preferredSize = [400, 150];
	exportPresets.selection = exportPresets.find(mainPDFExportPreset);
	var staticNotice = box.panel_options.add('statictext', undefined, "Выберите профиль [NONE], чтобы исключить эту операцию");
	setBaseProps(staticNotice);
	var c_Margin = box.panel_options.add("checkbox", undefined, "Включить размер страницы в имя файла");
	c_Margin.onClick = function () { c_Margin.value ? v_Margin = 1 : v_Margin = 0; }
	if (v_Margin == 1) c_Margin.value = true;
	setBaseProps(c_Margin);
	
	box.panel_merge = box.add('panel', undefined, "Слияние: экспорт всех именованных страниц в один файл");
	
	var mergeCaption = box.panel_merge.add('statictext', undefined, "Профиль для объединенного документа:");
	setBaseProps(mergeCaption);
	
	var mergePresets = box.panel_merge.add('listbox', undefined, refineDomArray(app.pdfExportPresets));
	mergePresets.selection = mergePresets.find(mergePDFExportPreset);
	mergePresets.preferredSize = [400, 150];
	var mergeNotice = box.panel_merge.add('statictext', undefined, "Выберите профиль [NONE], чтобы исключить эту операцию");
	setBaseProps(mergeNotice);

	box.panel_push = box.add('panel', undefined, "");
	var c_AllDocs = box.panel_push.add("checkbox", undefined, "Выполнить операцию для всех открытых документов");
	c_AllDocs.onClick = function () { c_AllDocs.value ? v_AllDocs = 1 : v_AllDocs = 0; }
	if (v_AllDocs == 1) c_AllDocs.value = true;
	setBaseProps(c_AllDocs);
	box.panel_push.okBtn = box.panel_push.add('button', undefined, "Старт", "");
	box.panel_push.okBtn.preferredSize = [400, 20];
	box.panel_push.okBtn.onClick = function () {
		exportPath = inputBox.text.replace(/\\/g, '/');
		if (!exportPath){ alert("Не выбрана папка экспорта"); return; }
		
		mainPDFExportPreset = app.pdfExportPresets.itemByName(String(exportPresets.selection));
		if (mainPDFExportPreset.isValid) { v_Ready = 1; }
		mergePDFExportPreset = app.pdfExportPresets.itemByName(String(mergePresets.selection));
		if (mergePDFExportPreset.isValid) { v_Ready = 1; }

		if (v_Ready == 1) {
			box.close();
			if (v_AllDocs == 1) {
				app.documents.everyItem().getElements().forEach( function (doc) {
					processDocument(doc);
				});
			}
			else
				processDocument(app.activeDocument);
		}
		else {
			alert("Профиль [NONE] выбран и для постраничного экспорта, и для слияния. Экспорт невозможен");
		}
	}

	box.show();
}

function processDocument(myDoc) {

	saveData ({
		v_Margin: v_Margin,
		v_Overwrite: v_Overwrite,
		v_Merge: v_Merge,
		v_AllDocs: v_AllDocs,
		mainPDFExportPreset: (mainPDFExportPreset.isValid ? mainPDFExportPreset.name : "[NONE]"),
		mergePDFExportPreset: (mergePDFExportPreset.isValid ? mergePDFExportPreset.name : "[NONE]"),
		exportPath: exportPath
	});
	
	var w = new Window ("palette", "Прогресс операций: " + myDoc.name);
	var pbar = w.add ("progressbar", undefined, 1, myDoc.pages.length);
	pbar.preferredSize = [500,20];
	w.show();
	
	var cLabelRangeSize = collectPageData(myDoc);
	if (!cLabelRangeSize) {
		alert("В документе нет страниц, отмеченных Script Label");
		return;
	}
	
	var f = new Folder(exportPath);
	if (!f.exists)
		f.create();
	
	if (mainPDFExportPreset.isValid) {
		cLabelRangeSize.forEach( function (LabelRangeSize) {
			var label = LabelRangeSize[0];
			var pageRange = LabelRangeSize[1];
			var pageSizeCollection = LabelRangeSize[2];
			var pageSizeList = "";
			pageSizeCollection.forEach( function (pageSizeItem) {
				pageSizeList = pageSizeList + (pageSizeList ? "," : "") + pageSizeItem;
			});
			
			app.pdfExportPreferences.pageRange = pageRange;
			
			fullName = exportPath + "/" + label;
			if (v_Margin == 1){ fullName += " - " + pageSizeList + " " + getUnits(); }
			fullName += ".pdf"; 
			
			var fullFile = File(fullName);
			if (!fullFile.exists || v_Overwrite) { myDoc.exportFile(ExportFormat.pdfType, new File(fullName), false, mainPDFExportPreset); }
			pbar.value ++;
		});
	}
	
	if (mergePDFExportPreset.isValid) {
		var mergePageRange = "";
		cLabelRangeSize.forEach( function (LabelRangeSize) {
			var pageRange = LabelRangeSize[1];
			mergePageRange = mergePageRange + (mergePageRange ? "," : "") + pageRange;
		});
		app.pdfExportPreferences.pageRange = mergePageRange;
		var fullName = exportPath + "/" + myDoc.name.toString().substr(0, myDoc.name.toString().lastIndexOf('.'));
		fullName += " - merged.pdf";
		
		var fullFile = File(fullName);
		if (!fullFile.exists || v_Overwrite) { myDoc.exportFile(ExportFormat.pdfType, new File(fullName), false, mergePDFExportPreset); }
	}
	
    w.close();
}

function collectPageData(doc) {
	var ret = [];
	var thisPageSize;
	
	doc.pages.everyItem().getElements().forEach( function (page) {
		if (page.label){
			if (v_Margin == 1){
				var pageBounds = page.bounds;
				thisPageSize = Math.round(pageBounds[3]-pageBounds[1]) + "x" + Math.round(pageBounds[2]-pageBounds[0]);
			}
			
			var i = indexOfCustom2d(page.label, ret, 0);
			if (i !== undefined) {
				ret[i][1] += "," + page.name;
				var j = indexOfCustom1d(thisPageSize, ret[i][2])
				if (j == undefined) { ret[i][2].push(thisPageSize); }
			}
			else { ret.push([page.label, page.name, [thisPageSize]]); }	
		}
	});
	
	if (ret.length)
		return ret;
}

function indexOfCustom2d(value, array, column) { // iterating 2d arrays
	for (var i = 0; i < array.length; i++) {
		if (array[i][column] === value) return i;
	}
}

function indexOfCustom1d(value, array) { // iterating 1d arrays
	for (var i = 0; i < array.length; i++) {
		if (array[i] === value) return i;
	}
}

function ClipGet(){  
	var clipboard;
	var script = 'Set objHTML = CreateObject("htmlfile")\r'+  
	'returnValue = objHTML.ParentWindow.ClipboardData.GetData("text")';  
	clipboard = app.doScript(script,ScriptLanguage.VISUAL_BASIC);
	return clipboard;
}

function setBaseProps(obj) {
	with (obj) {
		preferredSize = [400, 20];
		orientation = "row";
		alignment = "left";
	}
}

function refineDomArray(a) {
    var list = [];
    var item = [];
    for (i = 0; i < a.length; i++) {
        item = a[i].name;
        list.push(item);
    }
	list.unshift("[NONE]");
    return list;
}

function getUnits() {  
	var ret;
	switch (app.activeDocument.viewPreferences.horizontalMeasurementUnits) {
		case MeasurementUnits.MILLIMETERS : ret = 'мм' break;
		case MeasurementUnits.CENTIMETERS : ret = 'см' break;
		case MeasurementUnits.POINTS : ret = 'pt' break;
		case MeasurementUnits.PIXELS : ret = 'px' break;
		case MeasurementUnits.PICAS : ret = 'pc' break;
		case MeasurementUnits.INCHES : ret = 'in' break;
	}  
	return ret;
}

function saveData (obj) {
	var f = File (scriptPath().fullName.replace (/\.jsx?(bin)?$/, '.txt'));
	f.open ('w');
	f.write (obj.toSource());
	f.close ();
}

function getPrevious () {
	var f = File (scriptPath().fullName.replace (/\.jsx?(bin)?$/, '.txt'));
	if (f.exists){
		return $.evalFile(f);
	}
	else return 0;
}

function scriptPath () {
	try {
		return app.activeScript;
	} catch (e) {
		return File (e.fileName);
	}
}

function polyfillForEach () { // Mozilla minified polyfill adding .forEach() method to plain JS arrays
	if(!Array.prototype.forEach){Array.prototype.forEach=function(callback,thisArg){var T,k;if(this==null){throw new TypeError(' this is null or not defined');}
	var O=Object(this);var len=O.length>>>0;if(typeof callback!=='function'){throw new TypeError(callback+' is not a function');}
	if(arguments.length>1){T=thisArg;}
	k=0;while(k<len){var kValue;if(k in O){kValue=O[k];callback.call(T,kValue,k,O);}
	k++;}};}
}